@@ -20,7 +20,7 @@ struct can_edit<Content, std::void_t<decltype(Content::relations)>>
 
 template<class Content>
 [[gnu::used, gnu::retain]] void
-to_json(json &obj, const Event<Content> &event)
+to_json(nlohmann::json &obj, const Event<Content> &event)
 {
     obj["content"] = event.content;
     obj["sender"]  = event.sender;
@@ -32,7 +32,7 @@ to_json(json &obj, const Event<Content> &event)
 
 template<class Content>
 [[gnu::used, gnu::retain]] void
-from_json(const json &obj, Event<Content> &event)
+from_json(const nlohmann::json &obj, Event<Content> &event)
 {
     if (!std::is_same_v<Content, mtx::events::msg::Encrypted> &&
         obj.at("content").contains("m.new_content")) {
@@ -48,8 +48,10 @@ from_json(const json &obj, Event<Content> &event)
               obj.at("content").at("im.nheko.relations.v1.relations");
 
         event.content = new_content.get<Content>();
-    } else {
+    } else if (obj.at("content").is_object()) {
         event.content = obj.at("content").get<Content>();
+    } else {
+        event.content = {};
     }
 
     event.type   = getEventType(obj.at("type").get<std::string>());
@@ -61,18 +63,18 @@ from_json(const json &obj, Event<Content> &event)
 
 template<class Content>
 [[gnu::used, gnu::retain]] void
-from_json(const json &obj, DeviceEvent<Content> &event)
+from_json(const nlohmann::json &obj, DeviceEvent<Content> &event)
 {
     Event<Content> base_event = event;
     from_json(obj, base_event);
     event.content = base_event.content;
     event.type    = base_event.type;
-    event.sender  = obj.at("sender");
+    event.sender  = obj.at("sender").get<std::string>();
 }
 
 template<class Content>
 [[gnu::used, gnu::retain]] void
-to_json(json &obj, const DeviceEvent<Content> &event)
+to_json(nlohmann::json &obj, const DeviceEvent<Content> &event)
 {
     Event<Content> base_event = event;
     to_json(obj, base_event);
@@ -81,7 +83,7 @@ to_json(json &obj, const DeviceEvent<Content> &event)
 }
 
 void
-from_json(const json &obj, UnsignedData &data)
+from_json(const nlohmann::json &obj, UnsignedData &data)
 {
     if (obj.find("age") != obj.end())
         data.age = obj.at("age").get<uint64_t>();
@@ -104,7 +106,7 @@ from_json(const json &obj, UnsignedData &data)
 }
 
 void
-to_json(json &obj, const UnsignedData &event)
+to_json(nlohmann::json &obj, const UnsignedData &event)
 {
     if (!event.prev_sender.empty())
         obj["prev_sender"] = event.prev_sender;
@@ -127,17 +129,17 @@ to_json(json &obj, const UnsignedData &event)
 
 template<class Content>
 [[gnu::used, gnu::retain]] void
-from_json(const json &obj, StrippedEvent<Content> &event)
+from_json(const nlohmann::json &obj, StrippedEvent<Content> &event)
 {
     Event<Content> &base = event;
     from_json(obj, base);
 
-    event.state_key = obj.at("state_key");
+    event.state_key = obj.at("state_key").get<std::string>();
 }
 
 template<class Content>
 [[gnu::used, gnu::retain]] void
-to_json(json &obj, const StrippedEvent<Content> &event)
+to_json(nlohmann::json &obj, const StrippedEvent<Content> &event)
 {
     Event<Content> base_event = event;
     to_json(obj, base_event);
@@ -147,25 +149,25 @@ to_json(json &obj, const StrippedEvent<Content> &event)
 
 template<class Content>
 [[gnu::used, gnu::retain]] void
-from_json(const json &obj, RoomEvent<Content> &event)
+from_json(const nlohmann::json &obj, RoomEvent<Content> &event)
 {
     Event<Content> &base = event;
     from_json(obj, base);
 
-    event.event_id         = obj.at("event_id");
-    event.origin_server_ts = obj.at("origin_server_ts");
+    event.event_id         = obj.at("event_id").get<std::string>();
+    event.origin_server_ts = obj.at("origin_server_ts").get<uint64_t>();
 
     // SPEC_BUG: Not present in the state array returned by /sync.
     if (obj.find("room_id") != obj.end())
-        event.room_id = obj.at("room_id");
+        event.room_id = obj.at("room_id").get<std::string>();
 
     if (obj.find("unsigned") != obj.end())
-        event.unsigned_data = obj.at("unsigned");
+        event.unsigned_data = obj.at("unsigned").get<UnsignedData>();
 }
 
 template<class Content>
 [[gnu::used, gnu::retain]] void
-to_json(json &obj, const RoomEvent<Content> &event)
+to_json(nlohmann::json &obj, const RoomEvent<Content> &event)
 {
     Event<Content> base_event = event;
     to_json(obj, base_event);
@@ -180,7 +182,7 @@ to_json(json &obj, const RoomEvent<Content> &event)
 
 template<class Content>
 [[gnu::used, gnu::retain]] void
-to_json(json &obj, const StateEvent<Content> &event)
+to_json(nlohmann::json &obj, const StateEvent<Content> &event)
 {
     RoomEvent<Content> base_event = event;
     to_json(obj, base_event);
@@ -190,7 +192,7 @@ to_json(json &obj, const StateEvent<Content> &event)
 
 template<class Content>
 [[gnu::used, gnu::retain]] void
-from_json(const json &obj, StateEvent<Content> &event)
+from_json(const nlohmann::json &obj, StateEvent<Content> &event)
 {
     RoomEvent<Content> &base = event;
     from_json(obj, base);
@@ -200,7 +202,7 @@ from_json(const json &obj, StateEvent<Content> &event)
 
 template<class Content>
 [[gnu::used, gnu::retain]] void
-to_json(json &obj, const RedactionEvent<Content> &event)
+to_json(nlohmann::json &obj, const RedactionEvent<Content> &event)
 {
     RoomEvent<Content> base_event = event;
     to_json(obj, base_event);
@@ -210,7 +212,7 @@ to_json(json &obj, const RedactionEvent<Content> &event)
 
 template<class Content>
 [[gnu::used, gnu::retain]] void
-from_json(const json &obj, RedactionEvent<Content> &event)
+from_json(const nlohmann::json &obj, RedactionEvent<Content> &event)
 {
     RoomEvent<Content> &base = event;
     from_json(obj, base);
@@ -220,7 +222,7 @@ from_json(const json &obj, RedactionEvent<Content> &event)
 
 template<class Content>
 [[gnu::used, gnu::retain]] void
-to_json(json &obj, const EncryptedEvent<Content> &event)
+to_json(nlohmann::json &obj, const EncryptedEvent<Content> &event)
 {
     RoomEvent<Content> base_event = event;
     to_json(obj, base_event);
@@ -228,7 +230,7 @@ to_json(json &obj, const EncryptedEvent<Content> &event)
 
 template<class Content>
 [[gnu::used, gnu::retain]] void
-from_json(const json &obj, EncryptedEvent<Content> &event)
+from_json(const nlohmann::json &obj, EncryptedEvent<Content> &event)
 {
     RoomEvent<Content> &base = event;
     from_json(obj, base);
@@ -236,7 +238,7 @@ from_json(const json &obj, EncryptedEvent<Content> &event)
 
 template<class Content>
 [[gnu::used, gnu::retain]] void
-to_json(json &obj, const EphemeralEvent<Content> &event)
+to_json(nlohmann::json &obj, const EphemeralEvent<Content> &event)
 {
     obj["content"] = event.content;
     if constexpr (std::is_same_v<Unknown, Content>)
@@ -250,7 +252,7 @@ to_json(json &obj, const EphemeralEvent<Content> &event)
 
 template<class Content>
 [[gnu::used, gnu::retain]] void
-from_json(const json &obj, EphemeralEvent<Content> &event)
+from_json(const nlohmann::json &obj, EphemeralEvent<Content> &event)
 {
     event.content = obj.at("content").get<Content>();
     event.type    = getEventType(obj.at("type").get<std::string>());
