@@ -14,9 +14,6 @@
 #include <nlohmann/json.hpp>
 #endif
 
-using json       = nlohmann::json;
-namespace common = mtx::common;
-
 namespace mtx {
 //! Namespace for request structs
 namespace requests {
@@ -52,10 +49,25 @@ struct CreateRoom
     Preset preset = Preset::PrivateChat;
     //! Whether or not the room will be visible by non members.
     common::RoomVisibility visibility = common::RoomVisibility::Private;
+
+    //! A list of state events to set in the new room. This allows the user to override the default
+    //! state events set in the new room. The expected format of the state events are an object with
+    //! type, state_key and content keys set.
+    std::vector<events::collections::StrippedEvents> initial_state;
+
+    //! The room version to set for the room. If not provided, the homeserver is to use its
+    //! configured default. If provided, the homeserver will return a 400 error with the errcode
+    //! M_UNSUPPORTED_ROOM_VERSION if it does not support the room version.
+    std::string room_version;
+
+    //! Extra keys, such as m.federate, to be added to the content of the m.room.create event. The
+    //! server will overwrite the following keys: creator, room_version. Future versions of the
+    //! specification may allow the server to overwrite other keys.
+    std::optional<events::state::Create> creation_content;
 };
 
 void
-to_json(json &obj, const CreateRoom &request);
+to_json(nlohmann::json &obj, const CreateRoom &request);
 
 namespace login_identifier {
 //! The user is identified by their Matrix ID.
@@ -109,7 +121,7 @@ struct Login
 };
 
 void
-to_json(json &obj, const Login &request);
+to_json(nlohmann::json &obj, const Login &request);
 
 /// @brief Request payload for
 /// `POST /_matrix/client/r0/{register,account/password}/email/requestToken`
@@ -135,7 +147,7 @@ struct RequestEmailToken
 };
 
 void
-to_json(json &obj, const RequestEmailToken &request);
+to_json(nlohmann::json &obj, const RequestEmailToken &request);
 
 /// @brief Request payload for
 /// `POST /_matrix/client/r0/{register,account/password}/msisdn/requestToken`
@@ -163,7 +175,7 @@ struct RequestMSISDNToken
 };
 
 void
-to_json(json &obj, const RequestMSISDNToken &request);
+to_json(nlohmann::json &obj, const RequestMSISDNToken &request);
 
 //! Validate ownership of an email address/phone number.
 struct IdentitySubmitToken
@@ -177,7 +189,7 @@ struct IdentitySubmitToken
 };
 
 void
-to_json(json &obj, const IdentitySubmitToken &request);
+to_json(nlohmann::json &obj, const IdentitySubmitToken &request);
 
 //! Request payload for the `PUT /_matrix/client/r0/sendToDevice/{eventType}/{transcationID}`
 template<typename EventContent>
@@ -190,7 +202,7 @@ struct AvatarUrl
 };
 
 void
-to_json(json &obj, const AvatarUrl &request);
+to_json(nlohmann::json &obj, const AvatarUrl &request);
 
 //! Request payload for the `PUT /_matrix/client/r0/profile/{userId}/displayname` endpoint.
 struct DisplayName
@@ -200,7 +212,7 @@ struct DisplayName
 };
 
 void
-to_json(json &obj, const DisplayName &request);
+to_json(nlohmann::json &obj, const DisplayName &request);
 
 //! Request payload for the `POST /_matrix/client/r0/rooms/{roomId}/invite` endpoint as well as ban,
 //! unban and kick.
@@ -213,7 +225,7 @@ struct RoomMembershipChange
 };
 
 void
-to_json(json &obj, const RoomMembershipChange &request);
+to_json(nlohmann::json &obj, const RoomMembershipChange &request);
 
 //! Request payload for the `PUT /_matrix/client/r0/rooms/{roomId}/typing/{userId}` endpoint.
 struct TypingNotification
@@ -225,7 +237,7 @@ struct TypingNotification
 };
 
 void
-to_json(json &obj, const TypingNotification &request);
+to_json(nlohmann::json &obj, const TypingNotification &request);
 
 //! Request payload for the `PUT /_matrix/client/r0/devices/{deviceId}` endpoint.
 struct DeviceUpdate
@@ -234,7 +246,7 @@ struct DeviceUpdate
 };
 
 void
-to_json(json &obj, const DeviceUpdate &request);
+to_json(nlohmann::json &obj, const DeviceUpdate &request);
 
 //! Request payload for the `PUT /_matrix/client/r0/directory/list/room/{roomId}` endpoint
 struct PublicRoomVisibility
@@ -245,7 +257,7 @@ struct PublicRoomVisibility
 };
 
 void
-to_json(json &obj, const PublicRoomVisibility &request);
+to_json(nlohmann::json &obj, const PublicRoomVisibility &request);
 
 struct PublicRoomsFilter
 {
@@ -278,13 +290,13 @@ struct PublicRooms
 };
 
 void
-to_json(json &obj, const PublicRooms &request);
+to_json(nlohmann::json &obj, const PublicRooms &request);
 
 struct Empty
 {};
 
 inline void
-to_json(json &, const Empty &)
+to_json(nlohmann::json &, const Empty &)
 {}
 
 using Logout = Empty;
@@ -293,12 +305,15 @@ struct SignedOneTimeKey
 {
     //! Required. The unpadded Base64-encoded 32-byte Curve25519 public key.
     std::string key;
+    //! When uploading a signed key, an additional fallback: true key should be included to denote
+    //! that the key is a fallback key.
+    bool fallback = false;
     //! Required. Signatures of the key object.
     //! The signature is calculated using the process described at Signing JSON.
     std::map<std::string, std::map<std::string, std::string>> signatures;
 };
 void
-to_json(json &obj, const SignedOneTimeKey &);
+to_json(nlohmann::json &obj, const SignedOneTimeKey &);
 
 struct UploadKeys
 {
@@ -309,10 +324,24 @@ struct UploadKeys
     //! The names of the properties should be in the format `<algorithm>:<key_id>`.
     //! The format of the key is determined by the key algorithm.
     std::map<std::string, std::variant<std::string, SignedOneTimeKey>> one_time_keys;
+
+    //! The public key which should be used if the device’s one-time keys are exhausted. The
+    //! fallback key is not deleted once used, but should be replaced when additional one-time keys
+    //! are being uploaded. The server will notify the client of the fallback key being used through
+    //! /sync.
+    //!
+    //! There can only be at most one key per algorithm uploaded, and the server will only persist
+    //! one key per algorithm.
+    //!
+    //! When uploading a signed key, an additional fallback: true key should be included to denote
+    //! that the key is a fallback key.
+    //!
+    //! May be absent if a new fallback key is not required.
+    std::map<std::string, std::variant<std::string, SignedOneTimeKey>> fallback_keys;
 };
 
 void
-to_json(json &obj, const UploadKeys &);
+to_json(nlohmann::json &obj, const UploadKeys &);
 
 constexpr uint64_t DEFAULT_DOWNLOAD_TIMEOUT = 10 * 1000; // 10 seconds
 
@@ -333,7 +362,7 @@ struct QueryKeys
 };
 
 void
-to_json(json &obj, const QueryKeys &);
+to_json(nlohmann::json &obj, const QueryKeys &);
 
 struct ClaimKeys
 {
@@ -344,7 +373,7 @@ struct ClaimKeys
 };
 
 void
-to_json(json &obj, const ClaimKeys &request);
+to_json(nlohmann::json &obj, const ClaimKeys &request);
 
 struct KeySignaturesUpload
 {
@@ -357,7 +386,7 @@ struct KeySignaturesUpload
 };
 
 void
-to_json(json &obj, const KeySignaturesUpload &req);
+to_json(nlohmann::json &obj, const KeySignaturesUpload &req);
 
 //! Upload cross signing keys
 struct DeviceSigningUpload
@@ -373,7 +402,7 @@ struct DeviceSigningUpload
 };
 
 void
-to_json(json &obj, const DeviceSigningUpload &req);
+to_json(nlohmann::json &obj, const DeviceSigningUpload &req);
 
 struct PusherData
 {
@@ -388,7 +417,7 @@ struct PusherData
 };
 
 void
-to_json(json &obj, const PusherData &data);
+to_json(nlohmann::json &obj, const PusherData &data);
 
 //! Request payload for the `POST /_matrix/client/r0/pushers/set` endpoint.
 struct SetPusher
@@ -419,6 +448,6 @@ struct SetPusher
 };
 
 void
-to_json(json &obj, const SetPusher &req);
+to_json(nlohmann::json &obj, const SetPusher &req);
 } // namespace requests
 } // namespace mtx
