@@ -51,6 +51,7 @@ struct PushersData;
 struct SetPushers;
 }
 namespace responses {
+struct Aliases;
 struct Available;
 struct AvatarUrl;
 struct ClaimKeys;
@@ -59,9 +60,6 @@ struct CreateRoom;
 struct Device;
 struct EventId;
 struct FilterId;
-struct GroupId;
-struct GroupProfile;
-struct JoinedGroups;
 struct KeyChanges;
 struct KeySignaturesUpload;
 struct Login;
@@ -393,9 +391,12 @@ public:
     //! room.
     void join_room(const std::string &room,
                    const std::vector<std::string> &via,
-                   Callback<mtx::responses::RoomId> cb);
+                   Callback<mtx::responses::RoomId> cb,
+                   const std::string &reason = "");
     //! Leave a room by its room_id.
-    void leave_room(const std::string &room_id, Callback<mtx::responses::Empty> cb);
+    void leave_room(const std::string &room_id,
+                    Callback<mtx::responses::Empty> cb,
+                    const std::string &reason = "");
     //! Knock on a room.
     void knock_room(const std::string &room_id,
                     const std::vector<std::string> &via,
@@ -589,6 +590,15 @@ public:
       const std::map<mtx::identifiers::User, std::map<std::string, EventContent>> &messages,
       ErrCallback callback);
 
+    //! Resolve the specified roomalias to a roomid.
+    void resolve_room_alias(const std::string &alias, Callback<mtx::responses::RoomId> cb);
+    //! Add an alias to a room.
+    void add_room_alias(const std::string &alias, const std::string &roomid, ErrCallback cb);
+    //! Delete an alias from a room.
+    void delete_room_alias(const std::string &alias, ErrCallback cb);
+    //! List the local aliases on the users server.
+    void list_room_aliases(const std::string &room_id, Callback<mtx::responses::Aliases> cb);
+
     //! Gets the visibility of a given room on the server's public room directory.
     void get_room_visibility(const std::string &room_id,
                              Callback<mtx::responses::PublicRoomVisibility> cb);
@@ -619,19 +629,6 @@ public:
                        size_t max_depth        = 0,
                        bool suggested_only     = false);
 
-    //
-    // Group related endpoints.
-    //
-
-    void create_group(const std::string &localpart, Callback<mtx::responses::GroupId> cb);
-    void joined_groups(Callback<mtx::responses::JoinedGroups> cb);
-    void group_profile(const std::string &group_id, Callback<mtx::responses::GroupProfile> cb);
-    void group_rooms(const std::string &group_id, Callback<nlohmann::json> cb);
-    void set_group_profile(const std::string &group_id,
-                           nlohmann::json &req,
-                           Callback<nlohmann::json> cb);
-    void add_room_to_group(const std::string &room_id, const std::string &group_id, ErrCallback cb);
-
     //
     // Device related endpoints.
     //
@@ -852,6 +849,9 @@ MTXCLIENT_SEND_STATE_EVENT_FWD(state::PowerLevels)
 MTXCLIENT_SEND_STATE_EVENT_FWD(state::Tombstone)
 MTXCLIENT_SEND_STATE_EVENT_FWD(state::Topic)
 MTXCLIENT_SEND_STATE_EVENT_FWD(state::Widget)
+MTXCLIENT_SEND_STATE_EVENT_FWD(state::policy_rule::UserRule)
+MTXCLIENT_SEND_STATE_EVENT_FWD(state::policy_rule::RoomRule)
+MTXCLIENT_SEND_STATE_EVENT_FWD(state::policy_rule::ServerRule)
 MTXCLIENT_SEND_STATE_EVENT_FWD(state::space::Child)
 MTXCLIENT_SEND_STATE_EVENT_FWD(state::space::Parent)
 MTXCLIENT_SEND_STATE_EVENT_FWD(msc2545::ImagePack)
@@ -883,10 +883,13 @@ MTXCLIENT_SEND_ROOM_MESSAGE_FWD(mtx::events::msg::Video)
 // MTXCLIENT_SEND_ROOM_MESSAGE(mtx::events::msg::KeyVerificationCancel)
 // MTXCLIENT_SEND_ROOM_MESSAGE(mtx::events::msg::KeyVerificationKey)
 // MTXCLIENT_SEND_ROOM_MESSAGE(mtx::events::msg::KeyVerificationMac)
-MTXCLIENT_SEND_ROOM_MESSAGE_FWD(mtx::events::msg::CallInvite)
-MTXCLIENT_SEND_ROOM_MESSAGE_FWD(mtx::events::msg::CallCandidates)
-MTXCLIENT_SEND_ROOM_MESSAGE_FWD(mtx::events::msg::CallAnswer)
-MTXCLIENT_SEND_ROOM_MESSAGE_FWD(mtx::events::msg::CallHangUp)
+MTXCLIENT_SEND_ROOM_MESSAGE_FWD(mtx::events::voip::CallInvite)
+MTXCLIENT_SEND_ROOM_MESSAGE_FWD(mtx::events::voip::CallCandidates)
+MTXCLIENT_SEND_ROOM_MESSAGE_FWD(mtx::events::voip::CallAnswer)
+MTXCLIENT_SEND_ROOM_MESSAGE_FWD(mtx::events::voip::CallHangUp)
+MTXCLIENT_SEND_ROOM_MESSAGE_FWD(mtx::events::voip::CallSelectAnswer)
+MTXCLIENT_SEND_ROOM_MESSAGE_FWD(mtx::events::voip::CallReject)
+MTXCLIENT_SEND_ROOM_MESSAGE_FWD(mtx::events::voip::CallNegotiate)
 
 #define MTXCLIENT_SEND_TO_DEVICE_FWD(Content)                                                      \
     extern template void mtx::http::Client::send_to_device<Content>(                               \
