@@ -1,5 +1,3 @@
-#include <iostream>
-
 #include <nlohmann/json.hpp>
 #include <utility>
 
@@ -10,6 +8,8 @@
 #include "mtxclient/crypto/types.hpp"
 #include "mtxclient/crypto/utils.hpp"
 
+#include "mtx/log.hpp"
+
 using json = nlohmann::json;
 using namespace mtx::crypto;
 
@@ -79,7 +79,8 @@ OlmClient::identity_keys() const
     if (ret == olm_error())
         throw olm_exception("identity_keys", account_.get());
 
-    return json::parse(std::string(tmp_buf.begin(), tmp_buf.end()));
+    return json::parse(std::string(tmp_buf.begin(), tmp_buf.end()))
+      .get<mtx::crypto::IdentityKeys>();
 }
 
 std::string
@@ -110,7 +111,7 @@ OlmClient::sign_identity_keys()
 }
 
 std::size_t
-OlmClient::generate_one_time_keys(std::size_t number_of_keys)
+OlmClient::generate_one_time_keys(std::size_t number_of_keys, bool generate_fallback)
 {
     const std::size_t nbytes =
       olm_account_generate_one_time_keys_random_length(account_.get(), number_of_keys);
@@ -123,6 +124,14 @@ OlmClient::generate_one_time_keys(std::size_t number_of_keys)
     if (ret == olm_error())
         throw olm_exception("generate_one_time_keys", account_.get());
 
+    if (generate_fallback) {
+        const std::size_t fnbytes = olm_account_generate_fallback_key_random_length(account_.get());
+        buf                       = create_buffer(fnbytes);
+        auto temp = olm_account_generate_fallback_key(account_.get(), buf.data(), buf.size());
+        if (temp == olm_error())
+            throw olm_exception("generate_fallback_keys", account_.get());
+    }
+
     return ret;
 }
 
@@ -136,18 +145,33 @@ OlmClient::one_time_keys()
     if (ret == olm_error())
         throw olm_exception("one_time_keys", account_.get());
 
-    return json::parse(std::string(buf.begin(), buf.end()));
+    return json::parse(std::string(buf.begin(), buf.end())).get<mtx::crypto::OneTimeKeys>();
+}
+
+mtx::crypto::OneTimeKeys
+OlmClient::unpublished_fallback_keys()
+{
+    auto fbuf = create_buffer(olm_account_unpublished_fallback_key_length(account_.get()));
+
+    const auto fret =
+      olm_account_unpublished_fallback_key(account_.get(), fbuf.data(), fbuf.size());
+    if (fret == olm_error())
+        throw olm_exception("unpublished_fallback_keys", account_.get());
+
+    return json::parse(std::string(fbuf.begin(), fbuf.end())).get<mtx::crypto::OneTimeKeys>();
 }
 
 std::string
-OlmClient::sign_one_time_key(const std::string &key)
+OlmClient::sign_one_time_key(const std::string &key, bool fallback)
 {
     json j{{"key", key}};
+    if (fallback)
+        j["fallback"] = true;
     return sign_message(j.dump());
 }
 
 std::map<std::string, mtx::requests::SignedOneTimeKey>
-OlmClient::sign_one_time_keys(const OneTimeKeys &keys)
+OlmClient::sign_one_time_keys(const OneTimeKeys &keys, bool fallback)
 {
     // Sign & append the one time keys.
     std::map<std::string, mtx::requests::SignedOneTimeKey> signed_one_time_keys;
@@ -155,20 +179,21 @@ OlmClient::sign_one_time_keys(const OneTimeKeys &keys)
         const auto key_id       = elem.first;
         const auto one_time_key = elem.second;
 
-        auto sig = sign_one_time_key(one_time_key);
+        auto sig = sign_one_time_key(one_time_key, fallback);
 
         signed_one_time_keys["signed_curve25519:" + key_id] =
-          signed_one_time_key(one_time_key, sig);
+          signed_one_time_key(one_time_key, sig, fallback);
     }
 
     return signed_one_time_keys;
 }
 
 mtx::requests::SignedOneTimeKey
-OlmClient::signed_one_time_key(const std::string &key, const std::string &signature)
+OlmClient::signed_one_time_key(const std::string &key, const std::string &signature, bool fallback)
 {
     mtx::requests::SignedOneTimeKey sign{};
     sign.key        = key;
+    sign.fallback   = fallback;
     sign.signatures = {{user_id_, {{"ed25519:" + device_id_, signature}}}};
     return sign;
 }
@@ -176,11 +201,12 @@ OlmClient::signed_one_time_key(const std::string &key, const std::string &signat
 mtx::requests::UploadKeys
 OlmClient::create_upload_keys_request()
 {
-    return create_upload_keys_request(one_time_keys());
+    return create_upload_keys_request(one_time_keys(), unpublished_fallback_keys());
 }
 
 mtx::requests::UploadKeys
-OlmClient::create_upload_keys_request(const mtx::crypto::OneTimeKeys &one_time_keys)
+OlmClient::create_upload_keys_request(const mtx::crypto::OneTimeKeys &one_time_keys,
+                                      const mtx::crypto::OneTimeKeys &fallback_keys)
 {
     mtx::requests::UploadKeys req;
     req.device_keys.user_id   = user_id_;
@@ -196,14 +222,16 @@ OlmClient::create_upload_keys_request(const mtx::crypto::OneTimeKeys &one_time_k
 
     req.device_keys.signatures[user_id_]["ed25519:" + device_id_] = sig;
 
-    if (one_time_keys.curve25519.empty())
-        return req;
-
     // Sign & append the one time keys.
     auto temp = sign_one_time_keys(one_time_keys);
     for (const auto &[key_id, key] : temp)
         req.one_time_keys[key_id] = key;
 
+    temp = sign_one_time_keys(fallback_keys, true);
+    for (const auto &[key_id, key] : temp) {
+        req.fallback_keys[key_id] = key;
+    }
+
     return req;
 }
 
@@ -840,7 +868,7 @@ mtx::crypto::verify_identity_signature(const DeviceKeys &device_keys,
         return ed25519_verify_signature(signing_key, nlohmann::json(device_keys), signature);
 
     } catch (const nlohmann::json::exception &e) {
-        std::cerr << "verify_identity_signature: " << e.what();
+        mtx::utils::log::log()->error("verify_identity_signature: {}", e.what());
     }
 
     return false;
@@ -876,7 +904,7 @@ mtx::crypto::ed25519_verify_signature(std::string signing_key,
 
         return true;
     } catch (const nlohmann::json::exception &e) {
-        std::cerr << "verify_signature: " << e.what();
+        mtx::utils::log::log()->error("verify_signature: {}", e.what());
     }
 
     return false;
@@ -909,22 +937,18 @@ mtx::crypto::encrypt_exported_sessions(const mtx::crypto::ExportedSessionKeys &k
 
     // Format of the output buffer: (0x01 + salt + IV + number of rounds + ciphertext +
     // hmac-sha-256)
-    BinaryBuf output{0x01};
-    output.insert(
-      output.end(), std::make_move_iterator(salt.begin()), std::make_move_iterator(salt.end()));
-    output.insert(
-      output.end(), std::make_move_iterator(nonce.begin()), std::make_move_iterator(nonce.end()));
+    BinaryBuf output{
+      0x01,
+    };
+    output.insert(output.end(), salt.begin(), salt.end());
+    output.insert(output.end(), nonce.begin(), nonce.end());
     output.insert(output.end(), &iterationsArr[0], &iterationsArr[4]);
-    output.insert(output.end(),
-                  std::make_move_iterator(ciphertext.begin()),
-                  std::make_move_iterator(ciphertext.end()));
+    output.insert(output.end(), ciphertext.begin(), ciphertext.end());
 
     // Need to hmac-sha256 our string so far, and then use that to finish making the output.
     auto hmacSha256 = mtx::crypto::HMAC_SHA256(hmac256, output);
 
-    output.insert(output.end(),
-                  std::make_move_iterator(hmacSha256.begin()),
-                  std::make_move_iterator(hmacSha256.end()));
+    output.insert(output.end(), hmacSha256.begin(), hmacSha256.end());
     auto encrypted = std::string(output.begin(), output.end());
 
     return encrypted;
@@ -992,5 +1016,5 @@ mtx::crypto::decrypt_exported_sessions(const std::string &data, const std::strin
     auto decrypted = mtx::crypto::AES_CTR_256_Decrypt(ciphertext, aes256, iv);
 
     std::string plaintext(decrypted.begin(), decrypted.end());
-    return json::parse(plaintext);
+    return json::parse(plaintext).get<mtx::crypto::ExportedSessionKeys>();
 }
