@@ -1,7 +1,7 @@
 #include "mtx/requests.hpp"
 #include "mtx/events/collections.hpp"
 #include "mtx/events/encrypted.hpp"
-#include <iostream>
+
 #include <nlohmann/json.hpp>
 
 using json = nlohmann::json;
@@ -46,6 +46,22 @@ to_json(json &obj, const CreateRoom &request)
     obj["is_direct"]  = request.is_direct;
     obj["preset"]     = presetToString(request.preset);
     obj["visibility"] = visibilityToString(request.visibility);
+
+    if (!request.room_version.empty())
+        obj["room_version"] = request.room_version;
+
+    if (request.creation_content)
+        obj["creation_content"] = *request.creation_content;
+
+    if (!request.initial_state.empty()) {
+        auto arr = nlohmann::json::array();
+        for (const auto &ev : request.initial_state) {
+            auto event_json = std::visit([](auto e) { return json(e); }, ev);
+            event_json.erase("sender");
+            arr.push_back(std::move(event_json));
+        }
+        obj["initial_state"] = std::move(arr);
+    }
 }
 
 void
@@ -187,6 +203,8 @@ to_json(json &obj, const DeviceUpdate &request)
 void
 to_json(json &obj, const SignedOneTimeKey &request)
 {
+    if (request.fallback)
+        obj["fallback"] = true;
     obj["key"]        = request.key;
     obj["signatures"] = request.signatures;
 }
@@ -202,6 +220,10 @@ to_json(json &obj, const UploadKeys &request)
     for (const auto &[key_id, key] : request.one_time_keys) {
         obj["one_time_keys"][key_id] = std::visit([](const auto &e) { return json(e); }, key);
     }
+
+    for (const auto &[key_id, key] : request.fallback_keys) {
+        obj["fallback_keys"][key_id] = std::visit([](const auto &e) { return json(e); }, key);
+    }
 }
 
 void
