@@ -8,6 +8,7 @@
 #include "mtx/events/encrypted.hpp"
 #include "mtx/requests.hpp"
 #include "mtx/responses.hpp"
+#include "mtxclient/crypto/types.hpp"
 #include "mtxclient/http/client.hpp"
 
 #include "test_helpers.hpp"
@@ -164,7 +165,7 @@ TEST(ClientAPI, SSORedirect)
 {
     std::shared_ptr<Client> mtx_client = std::make_shared<Client>("localhost", 443);
     EXPECT_EQ(mtx_client->login_sso_redirect("http://aaa:555/sso"),
-              "https://localhost:443/_matrix/client/r0/login/sso/"
+              "https://localhost:443/_matrix/client/v3/login/sso/"
               "redirect?redirectUrl=http%3A%2F%2Faaa%3A555%2Fsso");
     mtx_client->close();
 }
@@ -289,6 +290,58 @@ TEST(ClientAPI, CreateRoom)
     mtx_client->close();
 }
 
+TEST(ClientAPI, CreateRoomInitialState)
+{
+    mtx::requests::CreateRoom req;
+
+    mtx::events::StrippedEvent<mtx::events::state::Encryption> enc;
+    enc.type                         = mtx::events::EventType::RoomEncryption;
+    enc.content.algorithm            = mtx::crypto::MEGOLM_ALGO;
+    enc.content.rotation_period_ms   = 1000ULL * 60ULL * 60ULL * 777ULL;
+    enc.content.rotation_period_msgs = 777;
+
+    req.initial_state.emplace_back(enc);
+
+    std::shared_ptr<Client> mtx_client = make_test_client();
+
+    mtx_client->login(
+      "alice", "secret", [mtx_client](const mtx::responses::Login &, RequestErr err) {
+          check_error(err);
+      });
+
+    while (mtx_client->access_token().empty())
+        sleep();
+
+    mtx_client->create_room(
+      req, [mtx_client, enc](const mtx::responses::CreateRoom &res, RequestErr err) {
+          check_error(err);
+          ASSERT_TRUE(res.room_id.localpart().size() > 10);
+          EXPECT_EQ(res.room_id.hostname(), server_name());
+
+          mtx_client->get_state(
+            res.room_id.to_string(), [enc](const mtx::responses::StateEvents &res, RequestErr err) {
+                check_error(err);
+                ASSERT_TRUE(res.events.size() > 0);
+                bool found_enc_event = false;
+
+                for (const auto &e : res.events) {
+                    auto ev =
+                      std::get_if<mtx::events::StateEvent<mtx::events::state::Encryption>>(&e);
+                    if (ev) {
+                        found_enc_event = true;
+                        EXPECT_EQ(ev->content.algorithm, enc.content.algorithm);
+                        EXPECT_EQ(ev->content.rotation_period_ms, enc.content.rotation_period_ms);
+                        EXPECT_EQ(ev->content.rotation_period_msgs,
+                                  enc.content.rotation_period_msgs);
+                    }
+                }
+                EXPECT_TRUE(found_enc_event);
+            });
+      });
+
+    mtx_client->close();
+}
+
 TEST(ClientAPI, Members)
 {
     std::shared_ptr<Client> mtx_client = make_test_client();
@@ -915,7 +968,7 @@ TEST(ClientAPI, Versions)
     mtx_client->versions([](const mtx::responses::Versions &res, RequestErr err) {
         check_error(err);
 
-        EXPECT_EQ(res.versions.size(), 7);
+        EXPECT_EQ(res.versions.size(), 10);
         EXPECT_EQ(res.versions.at(0), "r0.0.1");
         EXPECT_EQ(res.versions.at(1), "r0.1.0");
         EXPECT_EQ(res.versions.at(2), "r0.2.0");
@@ -923,6 +976,9 @@ TEST(ClientAPI, Versions)
         EXPECT_EQ(res.versions.at(4), "r0.4.0");
         EXPECT_EQ(res.versions.at(5), "r0.5.0");
         EXPECT_EQ(res.versions.at(6), "r0.6.0");
+        EXPECT_EQ(res.versions.at(7), "r0.6.1");
+        EXPECT_EQ(res.versions.at(8), "v1.1");
+        EXPECT_EQ(res.versions.at(9), "v1.2");
     });
 
     mtx_client->close();
@@ -1431,8 +1487,9 @@ TEST(ClientAPI, ReadMarkers)
                             std::get<mtx::events::EphemeralEvent<mtx::events::ephemeral::Receipt>>(
                               receipts.front())
                               .content.receipts[event_id];
-                          EXPECT_EQ(users.users.size(), 1);
-                          ASSERT_TRUE(users.users["@alice:" + server_name()].ts > 0);
+                          using mtx::events::ephemeral::Receipt;
+                          EXPECT_EQ(users[Receipt::Read].users.size(), 1);
+                          ASSERT_TRUE(users[Receipt::Read].users["@alice:" + server_name()].ts > 0);
                       });
                 });
           });
@@ -1452,19 +1509,19 @@ TEST(ClientAPI, SendToDevice)
     while (alice->access_token().empty() || bob->access_token().empty())
         sleep();
 
-    json body{{"messages",
-               {{bob->user_id().to_string(),
-                 {{bob->device_id(),
-                   {
-                     {"action", "request"},
-                     {"body",
-                      {{"sender_key", "test"},
-                       {"algorithm", "test_algo"},
-                       {"room_id", "test_room_id"},
-                       {"session_id", "test_session_id"}}},
-                     {"request_id", "test_request_id"},
-                     {"requesting_device_id", "test_req_id"},
-                   }}}}}}};
+    nlohmann::json body{{"messages",
+                         {{bob->user_id().to_string(),
+                           {{bob->device_id(),
+                             {
+                               {"action", "request"},
+                               {"body",
+                                {{"sender_key", "test"},
+                                 {"algorithm", "test_algo"},
+                                 {"room_id", "test_room_id"},
+                                 {"session_id", "test_session_id"}}},
+                               {"request_id", "test_request_id"},
+                               {"requesting_device_id", "test_req_id"},
+                             }}}}}}};
 
     alice->send_to_device("m.room_key_request", body, [bob](RequestErr err) {
         check_error(err);
@@ -1476,8 +1533,8 @@ TEST(ClientAPI, SendToDevice)
 
             EXPECT_EQ(res.to_device.events.size(), 1);
 
-            auto event =
-              std::get<mtx::events::DeviceEvent<msgs::KeyRequest>>(res.to_device.events[0]);
+            auto event = std::get<mtx::events::DeviceEvent<mtx::events::msg::KeyRequest>>(
+              res.to_device.events[0]);
             EXPECT_EQ(event.content.action, mtx::events::msg::RequestAction::Request);
             EXPECT_EQ(event.content.sender_key, "test");
             EXPECT_EQ(event.content.algorithm, "test_algo");
@@ -1508,10 +1565,10 @@ TEST(ClientAPI, NewSendToDevice)
            carl->access_token().empty())
         sleep();
 
-    ToDeviceMessages<msgs::KeyRequest> body1;
-    ToDeviceMessages<msgs::KeyRequest> body2;
+    ToDeviceMessages<mtx::events::msg::KeyRequest> body1;
+    ToDeviceMessages<mtx::events::msg::KeyRequest> body2;
 
-    msgs::KeyRequest request1;
+    mtx::events::msg::KeyRequest request1;
 
     request1.action               = mtx::events::msg::RequestAction::Request;
     request1.sender_key           = "test";
@@ -1523,7 +1580,7 @@ TEST(ClientAPI, NewSendToDevice)
 
     body1[bob->user_id()][bob->device_id()] = request1;
 
-    msgs::KeyRequest request2;
+    mtx::events::msg::KeyRequest request2;
 
     request2.action               = mtx::events::msg::RequestAction::Cancellation;
     request2.request_id           = "test_request_id_1";
@@ -1543,8 +1600,8 @@ TEST(ClientAPI, NewSendToDevice)
                 check_error(err);
 
                 EXPECT_EQ(res.to_device.events.size(), 2);
-                auto event =
-                  std::get<mtx::events::DeviceEvent<msgs::KeyRequest>>(res.to_device.events[0]);
+                auto event = std::get<mtx::events::DeviceEvent<mtx::events::msg::KeyRequest>>(
+                  res.to_device.events[0]);
             });
         });
     });
@@ -1601,113 +1658,6 @@ TEST(ClientAPI, RetrieveSingleEvent)
     bob->close();
 }
 
-TEST(Groups, Rooms)
-{
-    auto alice = make_test_client();
-    alice->login("alice", "secret", check_login);
-
-    WAIT_UNTIL(!alice->access_token().empty())
-
-    std::atomic<int> rooms_created(0);
-    std::vector<std::string> rooms_ids;
-
-    auto create_room_cb = [&rooms_created, &rooms_ids](const mtx::responses::CreateRoom &res,
-                                                       RequestErr err) {
-        check_error(err);
-        rooms_created += 1;
-        rooms_ids.push_back(res.room_id.to_string());
-    };
-
-    mtx::requests::CreateRoom req;
-    alice->create_room(req, create_room_cb);
-    alice->create_room(req, create_room_cb);
-
-    WAIT_UNTIL(rooms_created == 2)
-
-    auto random_group_id = to_string(random_number());
-
-    alice->create_group(
-      random_group_id,
-      [alice, rooms_ids, random_group_id](const mtx::responses::GroupId &res, RequestErr err) {
-          check_error(err);
-
-          EXPECT_EQ(res.group_id, std::string("+" + random_group_id + ":" + server_name()));
-
-          auto group_id = res.group_id;
-
-          alice->add_room_to_group(
-            rooms_ids.at(0), group_id, [alice, group_id, rooms_ids](RequestErr err) {
-                check_error(err);
-
-                alice->add_room_to_group(
-                  rooms_ids.at(1), group_id, [alice, rooms_ids, group_id](RequestErr err) {
-                      check_error(err);
-
-                      alice->joined_groups(
-                        [group_id](const mtx::responses::JoinedGroups &res, RequestErr err) {
-                            check_error(err);
-
-                            ASSERT_GE(res.groups.size(), 1);
-
-                            for (const auto &g : res.groups) {
-                                if (g == group_id)
-
-                                    return;
-                            }
-
-                            FAIL();
-                        });
-
-                      alice->group_rooms(group_id, [](const nlohmann::json &res, RequestErr err) {
-                          check_error(err);
-                          EXPECT_GE(res.at("chunk").size(), 2);
-                      });
-                  });
-            });
-      });
-
-    alice->close();
-}
-
-TEST(Groups, Profiles)
-{
-    auto alice = make_test_client();
-    alice->login("alice", "secret", check_login);
-
-    WAIT_UNTIL(!alice->access_token().empty())
-
-    auto random_group_id = to_string(random_number());
-    alice->create_group(
-      random_group_id,
-      [alice, random_group_id](const mtx::responses::GroupId &res, RequestErr err) {
-          check_error(err);
-
-          EXPECT_GE(res.group_id.size(), random_group_id.size());
-
-          // all of those keys need to be in the request on synapse
-          json profile;
-          profile["name"]              = "Name";
-          profile["avatar_url"]        = "";
-          profile["short_description"] = "short";
-          profile["long_description"]  = "long";
-          alice->set_group_profile(
-            "+" + random_group_id + ":" + server_name(),
-            profile,
-            [alice, random_group_id](const nlohmann::json &, RequestErr err) {
-                check_error(err);
-
-                alice->group_profile("+" + random_group_id + ":" + server_name(),
-                                     [](const mtx::responses::GroupProfile &res, RequestErr err) {
-                                         check_error(err);
-                                         EXPECT_EQ(res.name, "Name");
-                                         EXPECT_EQ(res.avatar_url, "");
-                                     });
-            });
-      });
-
-    alice->close();
-}
-
 TEST(ClientAPI, PublicRooms)
 {
     // Setup : Create a new (public) room with some settings
