@@ -27,7 +27,7 @@ TEST(RoomEvents, Reaction)
   "event_id": "$ujXAq1WXebS-vcpA4yBIZPvCeqGvnrMFP1c1qn8_wJump"
   })"_json;
 
-    RoomEvent<msg::Reaction> event = data;
+    RoomEvent<msg::Reaction> event = data.get<RoomEvent<msg::Reaction>>();
 
     EXPECT_EQ(event.type, EventType::Reaction);
     EXPECT_EQ(event.event_id, "$ujXAq1WXebS-vcpA4yBIZPvCeqGvnrMFP1c1qn8_wJump");
@@ -59,7 +59,7 @@ TEST(RoomEvents, Redacted)
           "type": "m.room.message"
         })"_json;
 
-    RoomEvent<msg::Redacted> event = data;
+    RoomEvent<msg::Redacted> event = data.get<RoomEvent<msg::Redacted>>();
 
     EXPECT_EQ(event.type, EventType::RoomMessage);
     EXPECT_EQ(event.event_id, "$143273582443PhrSn:localhost");
@@ -98,7 +98,7 @@ TEST(RoomEvents, AudioMessage)
           "type": "m.room.message"
         })"_json;
 
-    RoomEvent<msg::Audio> event = data;
+    RoomEvent<msg::Audio> event = data.get<RoomEvent<msg::Audio>>();
 
     EXPECT_EQ(event.type, EventType::RoomMessage);
     EXPECT_EQ(event.event_id, "$143273582443PhrSn:localhost");
@@ -141,7 +141,7 @@ TEST(RoomEvents, EmoteMessage)
           "room_id": "!VaMCVKSVcyPtXbcMXh:matrix.org"
         })"_json;
 
-    RoomEvent<msg::Emote> event = data;
+    RoomEvent<msg::Emote> event = data.get<RoomEvent<msg::Emote>>();
 
     EXPECT_EQ(event.type, EventType::RoomMessage);
     EXPECT_EQ(event.event_id, "$15098786822025533uttji:matrix.org");
@@ -192,7 +192,7 @@ TEST(RoomEvents, FileMessage)
           "room_id": "!lfoDRlNFWlvOnvkBwQ:matrix.org"
         })"_json;
 
-    RoomEvent<msg::File> event = data;
+    RoomEvent<msg::File> event = data.get<RoomEvent<msg::File>>();
 
     EXPECT_EQ(event.type, EventType::RoomMessage);
     EXPECT_EQ(event.event_id, "$15104856072749611ERqhw:matrix.org");
@@ -283,7 +283,7 @@ TEST(RoomEvents, EncryptedImageMessage)
       "age": 1234
   }
 })"_json;
-    RoomEvent<msg::Image> event = data;
+    RoomEvent<msg::Image> event = data.get<RoomEvent<msg::Image>>();
 
     EXPECT_EQ(event.type, EventType::RoomMessage);
     EXPECT_EQ(event.event_id, "$143273582443PhrSn:example.org");
@@ -338,7 +338,7 @@ TEST(RoomEvents, ImageMessage)
           "room_id": "!cURbafjkfsMDVwdRDQ:matrix.org"
         })"_json;
 
-    RoomEvent<msg::Image> event = data;
+    RoomEvent<msg::Image> event = data.get<RoomEvent<msg::Image>>();
 
     EXPECT_EQ(event.type, EventType::RoomMessage);
     EXPECT_EQ(event.event_id, "$15105042942524OGmZm:kamax.io");
@@ -405,7 +405,7 @@ TEST(RoomEvents, ImageMessage)
           "room_id": "!cURbafjkfsMDVwdRDQ:matrix.org"
         })"_json;
 
-    event = data;
+    event = data.get<RoomEvent<msg::Image>>();
 
     EXPECT_EQ(event.type, EventType::RoomMessage);
     EXPECT_EQ(event.event_id, "$15105042942524OGmZm:kamax.io");
@@ -454,7 +454,7 @@ TEST(RoomEvents, NoticeMessage)
           "room_id": "!BPvgRcBVHzyFSlYkrg:matrix.org"
         })"_json;
 
-    RoomEvent<msg::Notice> event = data;
+    RoomEvent<msg::Notice> event = data.get<RoomEvent<msg::Notice>>();
 
     EXPECT_EQ(event.type, EventType::RoomMessage);
     EXPECT_EQ(event.event_id, "$15104358652239178iCnZy:matrix.org");
@@ -494,7 +494,7 @@ TEST(RoomEvents, TextMessage)
           "room_id": "!lfoDRlNFWlvOnvkBwQ:matrix.org"
          })"_json;
 
-    RoomEvent<msg::Text> event = data;
+    RoomEvent<msg::Text> event = data.get<RoomEvent<msg::Text>>();
 
     EXPECT_EQ(event.type, EventType::RoomMessage);
     EXPECT_EQ(event.event_id, "$15104893562785758wEgEU:matrix.org");
@@ -551,7 +551,7 @@ TEST(RoomEvents, VideoMessage)
           "type": "m.room.message"
         })"_json;
 
-    RoomEvent<msg::Video> event = data;
+    RoomEvent<msg::Video> event = data.get<RoomEvent<msg::Video>>();
 
     EXPECT_EQ(event.type, EventType::RoomMessage);
     EXPECT_EQ(event.event_id, "$143273582443PhrSn:localhost");
@@ -610,7 +610,7 @@ TEST(RoomEvents, Sticker)
           "type": "m.sticker"
 	})"_json;
 
-    Sticker event = data;
+    Sticker event = data.get<Sticker>();
 
     EXPECT_EQ(event.type, EventType::Sticker);
     EXPECT_EQ(event.event_id, "$WLGTSEFSEF:localhost");
@@ -644,7 +644,7 @@ TEST(RoomEvents, Sticker)
 	  "event_id": "masked",
 	  "room_id": "!masked:devture.com"
 	})"_json;
-    Sticker event2 = data2;
+    Sticker event2 = data2.get<Sticker>();
     EXPECT_EQ(event2.type, EventType::Sticker);
     EXPECT_EQ(event2.content.body, "");
     EXPECT_EQ(event2.content.url, "mxc://devture.com/OCPxxxxxxxxxxxxxxxx");
@@ -669,7 +669,7 @@ TEST(FormattedMessages, Deserialization)
           "room_id": "!BPvgRcBVHzyFSlYkrg:matrix.org"
         })"_json;
 
-    RoomEvent<msg::Notice> notice = notice_data;
+    RoomEvent<msg::Notice> notice = notice_data.get<RoomEvent<msg::Notice>>();
 
     EXPECT_EQ(notice.type, EventType::RoomMessage);
     EXPECT_EQ(notice.content.msgtype, "m.notice");
@@ -690,7 +690,7 @@ TEST(FormattedMessages, Deserialization)
           "room_id": "!lfoDRlNFWlvOnvkBwQ:matrix.org"
          })"_json;
 
-    RoomEvent<msg::Text> text = text_data;
+    RoomEvent<msg::Text> text = text_data.get<RoomEvent<msg::Text>>();
 
     EXPECT_EQ(text.content.msgtype, "m.text");
     EXPECT_EQ(text.content.format, "org.matrix.custom.html");
@@ -710,7 +710,7 @@ TEST(FormattedMessages, Deserialization)
           "room_id": "!VaMCVKSVcyPtXbcMXh:matrix.org"
         })"_json;
 
-    RoomEvent<msg::Emote> emote = emote_data;
+    RoomEvent<msg::Emote> emote = emote_data.get<RoomEvent<msg::Emote>>();
 
     EXPECT_EQ(emote.content.msgtype, "m.emote");
     EXPECT_EQ(emote.content.format, "org.matrix.custom.html");
@@ -742,7 +742,7 @@ TEST(RoomEvents, Encrypted)
 	    }
 	})"_json;
 
-    RoomEvent<msg::Encrypted> event = data;
+    RoomEvent<msg::Encrypted> event = data.get<RoomEvent<msg::Encrypted>>();
 
     std::cout << "*****" << std::endl;
     std::cout << data.dump(2) << std::endl;
@@ -768,152 +768,3 @@ TEST(RoomEvents, Encrypted)
 
     EXPECT_EQ(data, json(event));
 }
-
-TEST(RoomEvents, CallInvite)
-{
-    nlohmann::json data = R"({
-          "content": {
-            "call_id": "c1591052749788",
-            "offer": {
-              "sdp": "v=0\r\no=- 6584580628695956864 2 IN IP4 127.0.0.1[...]",
-              "type": "offer"
-            },
-            "version": 0,
-            "lifetime": 120000
-          },
-          "event_id": "$143273582443PhrSn:example.org",
-          "origin_server_ts": 1432735824653,
-          "room_id": "!jEsUZKDJdhlrceRyVU:example.org",
-          "sender": "@example:example.org",
-          "type": "m.call.invite",
-          "unsigned": {
-            "age": 1234
-          }
-        })"_json;
-
-    RoomEvent<msg::CallInvite> event = data;
-
-    EXPECT_EQ(event.event_id, "$143273582443PhrSn:example.org");
-    EXPECT_EQ(event.origin_server_ts, 1432735824653L);
-    EXPECT_EQ(event.room_id, "!jEsUZKDJdhlrceRyVU:example.org");
-    EXPECT_EQ(event.sender, "@example:example.org");
-    EXPECT_EQ(event.type, EventType::CallInvite);
-    EXPECT_EQ(event.unsigned_data.age, 1234);
-    EXPECT_EQ(event.content.call_id, "c1591052749788");
-    EXPECT_EQ(event.content.sdp, "v=0\r\no=- 6584580628695956864 2 IN IP4 127.0.0.1[...]");
-    EXPECT_EQ(event.content.version, "0");
-    EXPECT_EQ(event.content.lifetime, 120000);
-
-    EXPECT_EQ(data, json(event));
-}
-
-TEST(RoomEvents, CallCandidates)
-{
-    nlohmann::json data = R"({
-          "content": {
-            "call_id": "c1591052749788",
-            "candidates": [
-              {
-                "sdpMid": "audio",
-                "sdpMLineIndex": 0,
-                "candidate": "candidate:863018703 1 udp 2122260223 10.9.64.156 43670 typ host generation 0"
-              }
-            ],
-            "version": 0
-          },
-          "event_id": "$143273582443PhrSn:example.org",
-          "origin_server_ts": 1432735824653,
-          "room_id": "!jEsUZKDJdhlrceRyVU:example.org",
-          "sender": "@example:example.org",
-          "type": "m.call.candidates",
-          "unsigned": {
-            "age": 1234
-          }
-        })"_json;
-
-    RoomEvent<msg::CallCandidates> event = data;
-
-    EXPECT_EQ(event.event_id, "$143273582443PhrSn:example.org");
-    EXPECT_EQ(event.origin_server_ts, 1432735824653L);
-    EXPECT_EQ(event.room_id, "!jEsUZKDJdhlrceRyVU:example.org");
-    EXPECT_EQ(event.sender, "@example:example.org");
-    EXPECT_EQ(event.type, EventType::CallCandidates);
-    EXPECT_EQ(event.unsigned_data.age, 1234);
-    EXPECT_EQ(event.content.call_id, "c1591052749788");
-    EXPECT_EQ(event.content.candidates[0].sdpMid, "audio");
-    EXPECT_EQ(event.content.candidates[0].sdpMLineIndex, 0);
-    EXPECT_EQ(event.content.candidates[0].candidate,
-              "candidate:863018703 1 udp 2122260223 10.9.64.156 43670 typ host generation 0");
-    EXPECT_EQ(event.content.version, "0");
-
-    EXPECT_EQ(data, json(event));
-}
-
-TEST(RoomEvents, CallAnswer)
-{
-    nlohmann::json data = R"({
-          "content": {
-            "call_id": "c1591052749788",
-            "answer": {
-              "sdp": "v=0\r\no=- 6584580628695956864 2 IN IP4 127.0.0.1[...]",
-              "type": "answer"
-            },
-            "version": 0
-          },
-          "event_id": "$143273582443PhrSn:example.org",
-          "origin_server_ts": 1432735824653,
-          "room_id": "!jEsUZKDJdhlrceRyVU:example.org",
-          "sender": "@example:example.org",
-          "type": "m.call.answer",
-          "unsigned": {
-            "age": 1234
-          }
-        })"_json;
-
-    RoomEvent<msg::CallAnswer> event = data;
-
-    EXPECT_EQ(event.event_id, "$143273582443PhrSn:example.org");
-    EXPECT_EQ(event.origin_server_ts, 1432735824653L);
-    EXPECT_EQ(event.room_id, "!jEsUZKDJdhlrceRyVU:example.org");
-    EXPECT_EQ(event.sender, "@example:example.org");
-    EXPECT_EQ(event.type, EventType::CallAnswer);
-    EXPECT_EQ(event.unsigned_data.age, 1234);
-    EXPECT_EQ(event.content.call_id, "c1591052749788");
-    EXPECT_EQ(event.content.sdp, "v=0\r\no=- 6584580628695956864 2 IN IP4 127.0.0.1[...]");
-    EXPECT_EQ(event.content.version, "0");
-
-    EXPECT_EQ(data, json(event));
-}
-
-TEST(RoomEvents, CallHangUp)
-{
-    nlohmann::json data = R"({
-          "content": {
-            "call_id": "c1591052749788",
-            "version": 0,
-            "reason": "invite_timeout"
-          },
-          "event_id": "$143273582443PhrSn:example.org",
-          "origin_server_ts": 1432735824653,
-          "room_id": "!jEsUZKDJdhlrceRyVU:example.org",
-          "sender": "@example:example.org",
-          "type": "m.call.hangup",
-          "unsigned": {
-            "age": 1234
-          }
-        })"_json;
-
-    RoomEvent<msg::CallHangUp> event = data;
-
-    EXPECT_EQ(event.event_id, "$143273582443PhrSn:example.org");
-    EXPECT_EQ(event.origin_server_ts, 1432735824653L);
-    EXPECT_EQ(event.room_id, "!jEsUZKDJdhlrceRyVU:example.org");
-    EXPECT_EQ(event.sender, "@example:example.org");
-    EXPECT_EQ(event.type, EventType::CallHangUp);
-    EXPECT_EQ(event.unsigned_data.age, 1234);
-    EXPECT_EQ(event.content.call_id, "c1591052749788");
-    EXPECT_EQ(event.content.version, "0");
-    EXPECT_EQ(event.content.reason, msg::CallHangUp::Reason::InviteTimeOut);
-
-    EXPECT_EQ(data, json(event));
-}
